package com.dragonkode.testing;

import java.util.ArrayList;

/**<pre>==========================================================================
 * The base class of a test framework for new classes. It is meant to work with 
 * MethodTest and TestValueLogic to create comprehensive test suites for a class'
 * interface.
 * 
 * ClassTester is a Strategy pattern implementation. It embodies the logic of
 * the overall test by instantiating MethodTest subclasses and calling their
 * MethodTest.test() methods. It can be used to create any standard test values 
 * or provide a single test object to test() methods.
 * 
 * The test framework makes heavy use of unchecked raw types (ArrayLists) for 
 * passing values between test framework objects and to the class under testing.
 * This is deliberate. First, if a test fails because of type issues, it should 
 * fail in the class being tested. If the test framework provided type
 * checking, it could hide faults in the class being tested. Second, this gives 
 * the tester maximum freedom when passing values through the test framework and 
 * lets the test framework be as general as possible. 
 * 
 * So be warned: You are responsible for putting the right types in the ArrayLists
 * of values for your tests!
 * 
 * Usage:
 * > Add field initialized with instance of class to be tested
 * > Create instance of the ClassTester subclass for this unit test ( [CTsubclass] )
 * > If you want to standard values for each range to be used by multiple 
 *   method-level unit tests, call: 
 *     [CTsubclass].setNormalValues(new ArrayList(Arrays.asList([values]);
 *     [CTsubclass].setHighValues(new ArrayList(Arrays.asList([values]);
 *     [CTsubclass].setLowValues(new ArrayList(Arrays.asList([values]);
 *     [CTsubclass].setInvalidValues(new ArrayList(Arrays.asList([values]);
 *     [CTsubclass].setStandardValues();
 * > Identify class-level unit test
 * > Call each MethodTest subclass in your unit test
 * 
 * @see MethodTest
 * @see ValueTestLogic
 * @see NoValueTest
 * @see MultiValueTest
 * @author J. Bradley Edwards - brad.edwards@dragonkode.com
 * @author DragonKode | dragonkode.com
 * @date 2013-06-08
 * @version 1.0
 * (c) 2013 DragonKode. This software is covered by the 
 * MIT license: http://opensource.org/licenses/MIT.
 * 
 * Fields:
 * > standardValues: holds an ArrayList of test values for each range 
 *   (normal, high, low, invalid). 
 *   NOTE: Order is important; it must be N,H,L,I or the MethodTest 
 *   logics will break.
 * > normalValues: holds normal double values for standard tests 
 * > highValues: holds high double values for standard tests
 * > lowValues: holds low double values for standard tests
 * > invalidValues: holds invalid double values for standard tests
 * 
 * Methods: 
 * 	setStandardValues(): used to set standard double values for method tests
 *	main(String[] args): sets standard values then executes Circle test using
 *		a MethodTest instance for each Circle method 
 *==========================================================================</pre>*/
public abstract class ClassTester {

	/* === FIELDS === */
	/** ArrayLists for each ranges of test values */
	@SuppressWarnings("rawtypes")
	protected ArrayList standardValues = new ArrayList();
	
	/** Normal range test values */
 	@SuppressWarnings("rawtypes")
	protected ArrayList normalValues = new ArrayList();
	
 	/** High range test values */
	@SuppressWarnings("rawtypes")
	protected ArrayList highValues = new ArrayList();
	
	/** Low range test values */
	@SuppressWarnings("rawtypes")
	protected ArrayList lowValues = new ArrayList();
	
	/** Invalid test values */
	@SuppressWarnings("rawtypes")
	protected ArrayList invalidValues = new ArrayList();
	
	/* === METHODS === */
	/**<pre>
	 * Controller for unit testing a class.
	 * 
	 * Logic:
	 * > Set any standard values
	 * > Identify class test to user
	 * > Call MethodTest subclasses for each method level unit test
	 * @param args not used
	 </pre>
	 */
	public static void main(String[] args) {
	}
	
	/* === GETTERS AND SETTERS === */
	
	/** 
	 * Returns the standardValues field.
	 * @return the standardValues */
	@SuppressWarnings("rawtypes")
	protected ArrayList getStandardValues() {
		return standardValues;
	}
	
	/** Sets the standardValues field.
	 * @param standardValues the standardValues to set */
	@SuppressWarnings({ "unchecked" })
	protected void setStandardValues() {
		if (standardValues == null) {
			throw new RuntimeException("standardValues null in ClassTester");
		}
		standardValues.add(normalValues);
		standardValues.add(highValues);
		standardValues.add(lowValues);
		standardValues.add(invalidValues);
	}

	/** 
	 * Returns the normalValues field.
	 * @return the normalValues */
	@SuppressWarnings("rawtypes")
	public ArrayList getNormalValues() {
		return normalValues;
	}

	/** Sets the normalValues field.
	 * @param normalValues the normalValues to set */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void setNormalValues(ArrayList normalValues) {
		this.normalValues.addAll((normalValues));
	}

	/** 
	 * Returns the highValues field.
	 * @return the highValues */
	@SuppressWarnings("rawtypes")
	public ArrayList getHighValues() {
		return highValues;
	}

	/** Sets the highValues field.
	 * @param highValues the highValues to set */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void setHighValues(ArrayList highValues) {
		this.highValues.addAll(highValues);
	}

	/** 
	 * Returns the lowValues field.
	 * @return the lowValues */
	@SuppressWarnings("rawtypes")
	public ArrayList getLowValues() {
		return lowValues;
	}

	/** Sets the lowValues field.
	 * @param lowValues the lowValues to set */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void setLowValues(ArrayList lowValues) {
		this.lowValues.addAll(lowValues);
	}

	/** 
	 * Returns the invalidValues field.
	 * @return the invalidValues */
	@SuppressWarnings("rawtypes")
	public ArrayList getInvalidValues() {
		return invalidValues;
	}

	/** Sets the invalidValues field.
	 * @param invalidValues the invalidValues to set */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void setInvalidValues(ArrayList invalidValues) {
		this.invalidValues.addAll(invalidValues);
	}
}