package com.dragonkode.testing;

import java.util.ArrayList;

/**<pre>=======================================================================
 * Represents the logic of method tests in the ClassTester framework.
 * 
 * This abstract class is a Strategy pattern implementation that ensures each
 * method test has the functionality needed for the testing framework. 
 * Subclasses either accept standard test values or implement test values for a
 * given method. Subclasses then implement test control logic and hold
 * instances of ValueTestLogic for valid and invalid value tests of the target
 * method. 
 * 
 * MethodTest is built for reuse and to minimize re-factoring. To do so
 * it uses a lot of unchecked and unparameterized values. It is up to you to
 * ensure you set your value values properly. That said, the advantage is that
 * class cast errors break against the target method instead of inside the 
 * testing framework. This will show how your methods behave when clients 
 * behave badly.
 * 
 * Usage:
 * > Create a subclass of MethodTest for each method you want to test
 * > For a no value test:
 *   > Override setValidTest() and initialize validTest with a new
 *     ValueTestLogic instance
 *   > Override test()
 *     > Identify the tested method using a System.out.print statement
 *     > call setValidTest()
 *     > call NoValueTest.test(validTest)
 * > For a multi-value test:
 *   > Override setValidTest() and setInvalidTest() and initialize both with
 *     new ValueTestLogic instances (validTest and invalidTest respectively)
 *   > if not using standard cases, override initValueSet()
 *   > Override test()
 *     > Identify the tested method using a System.out.print statement
 *     > initialize test value sets with initCaseSet() if standardCases are 
 *       not passed from your ClassTester instance
 *       > If you want to let the tester input cases, call methods from a 
 *         Tester instance here
 *     > call setValidTest()
 *     > call setInvalidTest() (if applicable)
 *     > call MultiValueTest.test(validTest,invalidTest)
 *
 * @see ClassTester
 * @see ValueTestLogic
 * @see NoValueTest
 * @see MultiValueTest
 * @author J. Bradley Edwards | brad.edwards@dragonkode.com
 * @author DragonKode | dragonkode.com
 * @date June 15, 2013
 * @version 1.0
 * (c) 2013 DragonKode. This software is covered by the 
 * MIT license: http://opensource.org/licenses/MIT.
 *
 * Fields:
 * > VALUERANGES: value range constants for method tests
 * > valueSet: set of values to use in method test
 * > validTest: logic for tests using valid  values
 * > invalidTest: logic for tests using invalid values
 * > testFault: flag for single value failures in a method test
 * > testFail: flag for overall test fail
 * > tester: tester instance for getting values from tester
 * Methods:
 * > MethodTest() calls setValidTest() and setInvalidTest()
 * > MethodTest(ArrayList) passes ArrayList of values to initValueSet
 *   then calls setValidTest and setInvalidTests
 * > initValueSet() initializes test value set
 * > initValueSet(ArrayList) uses ArrayList to initialize test Value (value) set
 * > setValidTest() creates a TestLogic instance with valid value test logic
 * > setInvalidTest() creates a TestLogic instance with invalid value 
 *		test logic
 * > test() runs the method test(s)
 * ====================================================================</pre>*/
public abstract class MethodTest {
	/* === FIELDS === */
	/**	Value range constants for method tests */
	protected String[] VALUERANGES = 
		{"NORMAL", "HIGH", "LOW", "INVALID", "TESTER"};

	/** Set of Values to use in method test */
	@SuppressWarnings("rawtypes")
	protected ArrayList valueSet = new ArrayList();
	
	/** Logic for tests using valid values or no parameter tests */
	protected ValueTestLogic validTest;
	
	/** Logic for tests using invalid values */
	protected ValueTestLogic invalidTest;
	
	/** Flag for single value failures in a method test */
	protected boolean testFault = false;
	
	/** Flag for overall test fail */
	protected boolean testFail = false;
	
	/** Tester instances for getting arbitrary test values */
	protected Tester tester = new Tester();
	
	/** Values from tester */
	@SuppressWarnings("rawtypes")
	ArrayList testerValues;
	
	/* === METHODS === */
	/**
	 * Default constructor
	 */
	public MethodTest() {
	}

	/**<pre>
	 * Constructor that accepts a set of standard Value values
	 * 
	 * Logic:
	 * > pass standardValues to initValueSet(ArrayList)
	 * 
	 * @param standardValues ArrayList of values for each range of Values
	 </pre>*/
	@SuppressWarnings({ "rawtypes" })
	public MethodTest(ArrayList standardValues) {
		initValueSet(standardValues);
	}

	/**<pre>
	 * Initializes set of Value values for the method test and checks
	 * for Values from the tester.
	 * 
	 * Logic:
	 * > add Values for each range to valueSet (order: normal, high, low, invalid)
	 * > check for Values from tester
	 </pre>*/
	protected void initValueSet() {
	}

	/**<pre>
	 * Initializes set of Value values from an ArrayList
	 * 
	 * Logic:
	 * > set valueSet to Values
	 * 
	 * @param values ArrayList of standardValues
	 </pre>
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public void initValueSet(ArrayList values) {
		valueSet.addAll(values);
	}

	/**<pre>
	 * Creates a TestLogic instance representing logic for a method test
	 * using valid test values
	 *   
	 * Logic:
	 * > initialize validTest with new instance of TestLogic for 
	 *   valid value tests
	 </pre>*/
	protected void setValidTest() {
	}

	/**<pre>
	 * Creates a TestLogic instance representing logic for a method test
	 * using invalid test values
	 * 
	 * Logic:
	 * > initialize invalidTest with new instance of TestLogic for 
	 *   invalid value tests
	 </pre>*/
	protected void setInvalidTest() {
	}

	/**<pre>
	 * Initializes values set, tests, gets any values from tester and runs test.
	 * 
	 * Logic:
	 * > identify test to user with System.out.print("targetMethod(args): ")
	 * > if a Value set is needed, check if valueSet is null and run initValueSet if
	 *   needed
	 * > if appropriate, check if tester has values for unit test
	 *   > if tester has values, get and add them to valueSet
	 * > set test(s)
	 * > run test(s)
	 * 
	 * @see NoValueTest
	 * @see MultiValueTest
	 </pre>*/
	public void test() {
		System.out.println("Error: implement test()");
	}
}
