package com.dragonkode.testing;

import java.util.ArrayList;

/**<pre>=======================================================================
 * Represents the control logic of a unit test for a method that has parameters.
 *
 * Developer Note: This class could have been implemented with only one boolean.
 * testFault and testFail are included for future extensions. A planned feature 
 * is detailed logging of which values failed in which range. Both booleans are
 * needed to support that feature to allow the program to differentiate single
 * value faults from overall test failures.
 *  
 *  Usage:
 *  > call test(String[],ValueTestLogic,ValueTestLogic,ArrayList) inside 
 *  [MethodTest subclass].test()
 *    > where:
 *      > String[] holds constants for ranges of cases (i.e. normal, high, low,
 *      invalid, tester)
 *      > ValueTestLogic holds unit test logic for valid values
 *      > ValueTestLogic holds unit test logic for invalid values
 *      > ArrayList holds an ArrayList of values for each range of values in 
 *        the unit test
 *  
 * @see ClassTester
 * @see MethodTest
 * @see MultiValueTest
 * @see ValueTestLogic
 * @author J. Bradley Edwards | brad.edwards@dragonkode.com
 * @author DragonKode | dragonkode.com
 * @date June 21, 2013
 * @version 1.0
 * (c) 2013 DragonKode. This software is covered by the 
 * MIT license: http://opensource.org/licenses/MIT.
 *
 * Fields:
 * > testFault: indicates whether any values have failed the unit test
 * > testFail: indicates whether the method passed or failed the unit test
 *
 * Methods:
 * > test(String[], ValueTestLogic, ValueTestLogic, ArrayList): Runs the unit test 
 *   with the right test logic for all values each value range and outputs the
 *   result.
 *
 * ====================================================================</pre>*/
public class MultiValueTest {
	/* === FIELDS === */
	/** Indicates whether any values have failed the unit test */
	private static boolean testFault;
	
	/** Indicates whether the method passed or failed the unit test */
	private static boolean testFail;
	
	/* === METHODS === */
	/**<pre>
	 * Runs the unit test with the right test logic for all values each value 
	 * range and outputs the result.
	 * 
	 * Logic:
	 * > reset testFault and testFail
	 * > for each value range
	 *   > select the correct test logic for the range
	 *   > run all values in the range through the appropriate test
	 *   > if any value in the range faults, set testFail = true
	 * > report test result
	 * 
	 * @param RANGES range of values for the unit test
	 * @param valid unit test logic for valid values for a specific method
	 * @param invalid unit test logic for invalid values for a specific method
	 * @param valueSet set holds a set of values for each value range in the unit test
	 </pre>*/
	@SuppressWarnings("rawtypes")
	public static void test
			(String[] RANGES, ValueTestLogic valid, ValueTestLogic invalid,
			ArrayList valueSet) {
		/* reset test flags */
		testFault = false;
		testFail = false;		
		/* initialize utility fields */
		/** Range counter */
		int i =0;
		/** Counter for values in the current range */
		int j = 0;
		/** Unit test logic for current range */
		ValueTestLogic currentTest;
		/* for each value range, reset testFault, choose the correct test logic and test
		   each value in the range */
		for (i = 0; i < valueSet.size(); i++) {
			/* if the current range has no values, skip this range */
			if (valueSet.get(i) == null) {
				continue;
			}
			testFault = false;
			/* select the unit test logic for the current range */
			if (RANGES[i].equals("INVALID")) {
				currentTest = invalid;
			} else {
				currentTest = valid;
			}
			/* for every value in the range, test the value against the target method */
			for (j = 0; j < ((ArrayList) valueSet.get(i)).size(); j++) {
				testFault = currentTest.test(((ArrayList) valueSet.get(i)).get(j));
			}
			/* If the test fails with the current value, set testFail to true */
			if (testFault == true) {
				testFail = true;
			}
		}
		/* Report result of the method unit test */
		if (testFail == true) {
			System.out.print("FAIL\n");
		} else {
			System.out.print("OK\n");
		}
	}
}
